import argparse
import requests
import os
from pathlib import Path

MODEL_URLS = {
    "onnx": "https://huggingface.co/janakh/monocr/resolve/main/monocr.onnx",
    "tflite": "https://huggingface.co/janakh/monocr/resolve/main/monocr.tflite",
    "charset": "https://huggingface.co/janakh/monocr/resolve/main/charset.txt",
}

def download_file(url, dest):
    print(f"Downloading {url} to {dest}...")
    response = requests.get(url, stream=True)
    response.raise_for_status()
    
    os.makedirs(os.path.dirname(dest), exist_ok=True)
    with open(dest, "wb") as f:
        for chunk in response.iter_content(chunk_size=8192):
            f.write(chunk)
    print(f"Done.")

def main():
    parser = argparse.ArgumentParser(description="Download MonOCR models")
    parser.add_argument("--dest", type=str, default="model", help="Destination directory")
    args = parser.parse_args()
    
    dest_path = Path(args.dest)
    
    for name, url in MODEL_URLS.items():
        ext = ".txt" if name == "charset" else f".{name}"
        file_dest = dest_path / f"monocr{ext}" if name != "charset" else dest_path / "charset.txt"
        download_file(url, file_dest)

if __name__ == "__main__":
    main()



import click
from pathlib import Path
from .ocr import read_image, read_pdf
from .model_manager import ModelManager

@click.group()
def main():
    """MonOCR Command Line Interface"""
    pass

@main.command()
@click.argument('image_path', type=click.Path(exists=True))
@click.option('--model', '-m', default=None, help='Path to ONNX model (optional)')
@click.option('--charset', '-c', default=None, help='Path to charset file (optional)')
def image(image_path, model, charset):
    """Recognize text from an image file."""
    try:
        text = read_image(image_path, model, charset)
        click.echo(text)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        exit(1)

@main.command()
@click.argument('pdf_path', type=click.Path(exists=True))
@click.option('--model', '-m', default=None, help='Path to ONNX model (optional)')
@click.option('--charset', '-c', default=None, help='Path to charset file (optional)')
def pdf(pdf_path, model, charset):
    """Recognize text from a PDF file."""
    try:
        pages = read_pdf(pdf_path, model, charset)
        for i, text in enumerate(pages):
            click.echo(f"--- Page {i+1} ---")
            click.echo(text)
            click.echo()
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        exit(1)

@main.command()
@click.option('--force', '-f', is_flag=True, help='Force re-download')
def download(force):
    """Download model files to local cache."""
    manager = ModelManager()
    if force and (manager.cache_dir / manager.MODEL_FILENAME).exists():
        (manager.cache_dir / manager.MODEL_FILENAME).unlink()
        
    try:
        manager.get_model_path() # Triggers download if missing
        click.echo("Model is ready.")
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        exit(1)

@main.command()
@click.argument('directory', type=click.Path(exists=True, file_okay=False))
@click.option('--model', '-m', default=None, help='Path to ONNX model')
@click.option('--charset', '-c', default=None, help='Path to charset')
def batch(directory, model, charset):
    """Process all images in a directory."""
    path = Path(directory)
    extensions = {'.png', '.jpg', '.jpeg', '.bmp', '.tiff'}
    
    files = [f for f in path.iterdir() if f.suffix.lower() in extensions]
    
    if not files:
        click.echo("No images found in directory.")
        return

    from .predictor import MonOCR
    ocr = MonOCR(model, charset)
    
    for file in sorted(files):
        click.echo(f"Processing {file.name}...", err=True)
        try:
            text = ocr.predict(file)
            click.echo(f"--- {file.name} ---")
            click.echo(text)
            click.echo()
        except Exception as e:
             click.echo(f"Failed to process {file.name}: {e}", err=True)

if __name__ == '__main__':
    main()



from pathlib import Path
from monocr_onnx import read_pdf

project_root = Path(__file__).resolve().parent.parent.parent
pdf_path = project_root / "data/pdfs/Mon_E_library.pdf"

print(f"Reading PDF: {pdf_path}")
try:
    pages = read_pdf(pdf_path)
    for i, text in enumerate(pages):
        print(f"--- Page {i+1} ---")
        print(text)
        print()
except Exception as e:
    print(f"Error: {e}")



def levenshtein_distance(s1, s2):
    if len(s1) < len(s2):
        return levenshtein_distance(s2, s1)

    if len(s2) == 0:
        return len(s1)

    previous_row = range(len(s2) + 1)
    for i, c1 in enumerate(s1):
        current_row = [i + 1]
        for j, c2 in enumerate(s2):
            insertions = previous_row[j + 1] + 1
            deletions = current_row[j] + 1
            substitutions = previous_row[j] + (c1 != c2)
            current_row.append(min(insertions, deletions, substitutions))
        previous_row = current_row

    return previous_row[-1]

def calculate_accuracy(pred, truth):
    if not truth:
        return 0.0
    distance = levenshtein_distance(pred, truth)
    max_len = max(len(pred), len(truth))
    if max_len == 0:
        return 100.0
    return (1 - distance / max_len) * 100.0



from .ocr import read_image, read_images, read_pdf, read_pdfs, read_image_with_accuracy
from .predictor import MonOCR
from .model_manager import ModelManager

__version__ = "0.1.0"
__all__ = [
    "read_image",
    "read_images",
    "read_pdf", 
    "read_pdfs",
    "read_image_with_accuracy",
    "MonOCR",
    "ModelManager"
]



from pathlib import Path
from PIL import Image
from pdf2image import convert_from_path
from .predictor import MonOCR
from .utils import calculate_accuracy

def read_image(image_path, model_path=None, charset_path=None):
    """
    Recognize text from an image file.
    
    Args:
        image_path (str or Path): Path to the image file.
        model_path (str, optional): Path to the ONNX model. If None, uses auto-downloaded model.
        charset_path (str, optional): Path to the charset file. If None, uses bundled charset.
        
    Returns:
        str: Recognized text.
    """
    ocr = MonOCR(model_path, charset_path)
    return ocr.predict(image_path)

from concurrent.futures import ThreadPoolExecutor

def read_images(image_paths, model_path=None, charset_path=None, workers=4):
    """
    Recognize text from multiple image files in parallel.
    
    Args:
        image_paths (list[str or Path]): List of paths to image files.
        model_path (str, optional): Path to model.
        charset_path (str, optional): Path to charset.
        workers (int, optional): Number of parallel workers. Defaults to 4.
        
    Returns:
        list[str]: List of recognized text for each image.
    """
    ocr = MonOCR(model_path, charset_path)
    
    with ThreadPoolExecutor(max_workers=workers) as executor:
        results = list(executor.map(ocr.predict, image_paths))
        
    return results

def read_pdf(pdf_path, model_path=None, charset_path=None):
    """
    Recognize text from a PDF file.
    
    Args:
        pdf_path (str or Path): Path to the PDF file.
        model_path (str, optional): Path to model.
        charset_path (str, optional): Path to charset.
        
    Returns:
        list[str]: list of recognized text per page.
    """
    try:
        images = convert_from_path(str(pdf_path), dpi=300)
    except Exception as e:
        raise RuntimeError(f"Failed to convert PDF. Ensure poppler-utils is installed. Error: {e}")
    
    ocr = MonOCR(model_path, charset_path)
    # Process pages sequentially as they are in memory
    results = [ocr.predict(img) for img in images]
        
    return results

def read_pdfs(pdf_paths, model_path=None, charset_path=None, workers=4):
    """
    Recognize text from multiple PDF files.
    
    Args:
        pdf_paths (list[str or Path]): List of paths to PDF files.
        model_path (str, optional): Path to model.
        charset_path (str, optional): Path to charset.
        workers (int, optional): Workers for parallelism.
        
    Returns:
        list[list[str]]: List of lists of recognized text per page for each PDF.
    """
    def process_single_pdf(path):
        return read_pdf(path, model_path, charset_path)

    with ThreadPoolExecutor(max_workers=workers) as executor:
        results = list(executor.map(process_single_pdf, pdf_paths))
        
    return results

def read_image_with_accuracy(image_path, ground_truth, model_path=None, charset_path=None):
    """
    Recognize text and calculate accuracy against ground truth.
    
    Returns:
        tuple: (recognized_text, accuracy_percentage)
    """
    text = read_image(image_path, model_path, charset_path)
    accuracy = calculate_accuracy(text, ground_truth)
    return text, accuracy



import sys
from pathlib import Path
from monocr_onnx import read_image

# Path to test image
project_root = Path(__file__).resolve().parent.parent.parent
image_path = project_root / "data/images/test_0005_h71.png"

print(f"Reading image: {image_path}")
text = read_image(image_path)

print("Recognized Text:")
print("================")
print(text)
print("================")



import numpy as np
import onnxruntime as ort
from PIL import Image
from pathlib import Path
import importlib.resources
from .model_manager import ModelManager
from .segmenter import LineSegmenter

class MonOCR:
    def __init__(self, model_path=None, charset_path=None):
        if model_path is None:
            manager = ModelManager()
            model_path = manager.get_model_path()
            
        self.session = ort.InferenceSession(str(model_path), providers=['CPUExecutionProvider'])
        self.segmenter = LineSegmenter()
        
        if charset_path:
            with open(charset_path, 'r', encoding='utf-8') as f:
                self.charset = f.read().strip()
        else:
            # Load bundled charset
            try:
                # Python 3.9+ resource loading
                ref = importlib.resources.files('monocr_onnx') / 'charset.txt'
                with ref.open('r', encoding='utf-8') as f:
                    self.charset = f.read().strip()
            except Exception as e:
                # Fallback for older python or loose script usage
                charset_file = Path(__file__).parent / 'charset.txt'
                if charset_file.exists():
                     with open(charset_file, 'r', encoding='utf-8') as f:
                        self.charset = f.read().strip()
                else:
                    raise RuntimeError(f"Charset file not found. Error: {e}")

    def preprocess(self, img):
        if img.mode != 'L':
            img = img.convert('L')
        
        target_height = 64
        # Handle small images
        if img.height == 0:
             return None
             
        aspect_ratio = img.width / img.height
        target_width = int(target_height * aspect_ratio)
        img = img.resize((target_width, target_height), Image.Resampling.BILINEAR)
        
        img_arr = np.array(img).astype(np.float32) / 255.0
        img_arr = np.expand_dims(img_arr, axis=0) # Add channel dim
        img_arr = np.expand_dims(img_arr, axis=0) # Add batch dim
        return img_arr

    def decode(self, preds):
        idx2char = {i + 1: c for i, c in enumerate(self.charset)}
        decoded_text = []
        prev_idx = -1
        
        # Simple Greedy Decoding
        for idx in preds:
            if idx != 0 and idx != prev_idx:
                decoded_text.append(idx2char.get(idx, ""))
            prev_idx = idx
            
        return "".join(decoded_text)

    def predict_line(self, img):
        if isinstance(img, (str, Path)):
            img = Image.open(img)
            
        input_data = self.preprocess(img)
        if input_data is None:
            return ""
        
        input_name = self.session.get_inputs()[0].name
        output_name = self.session.get_outputs()[0].name
        
        outputs = self.session.run([output_name], {input_name: input_data})
        preds = np.argmax(outputs[0], axis=2)[0] # Batch size 1
        
        return self.decode(preds)

    def predict_page(self, img_path):
        """
        Segment page into lines and predict each line.
        """
        if isinstance(img_path, (str, Path)):
            img = Image.open(img_path)
        else:
            img = img_path

        lines = self.segmenter.segment(img)
        results = []
        
        if not lines:
            # Fallback: maybe it IS a single line?
            # Or return empty string?
            # Let's try to predict as single line if segmentation failed but image is smallish?
            # actually segmenter should handle single line if it has enough height
            # If segmentation returns nothing, it means empty page or very noisy.
            # But let's try single line prediction as fallback for robustness
            text = self.predict_line(img)
            if text:
                return text
            return ""

        for line in lines:
            text = self.predict_line(line['img'])
            results.append(text)
            
        return "\n".join(results)

    def predict(self, img_path):
        # Alias for backward compatibility or ease of use
        return self.predict_page(img_path)



import os
import requests
from pathlib import Path
from tqdm import tqdm

class ModelManager:
    MODEL_FILENAME = "monocr.onnx"
    MODEL_URL = "https://huggingface.co/janakhpon/monocr/resolve/main/onnx/monocr.onnx"
    
    def __init__(self):
        self.cache_dir = Path.home() / ".monocr" / "models"
        
    def get_model_path(self):
        model_path = self.cache_dir / self.MODEL_FILENAME
        
        if not model_path.exists():
            print(f"Model not found at {model_path}. Downloading...")
            self.download_model()
            
        return model_path
        
    def download_model(self):
        self.cache_dir.mkdir(parents=True, exist_ok=True)
        dest_path = self.cache_dir / self.MODEL_FILENAME
        
        try:
            response = requests.get(self.MODEL_URL, stream=True, allow_redirects=True)
            response.raise_for_status()
            
            total_size = int(response.headers.get('content-length', 0))
            
            with open(dest_path, "wb") as f, tqdm(
                desc=self.MODEL_FILENAME,
                total=total_size,
                unit='iB',
                unit_scale=True,
                unit_divisor=1024,
            ) as bar:
                for chunk in response.iter_content(chunk_size=8192):
                    size = f.write(chunk)
                    bar.update(size)
                    
            print(f"Model downloaded successfully to {dest_path}")
            
        except Exception as e:
            if dest_path.exists():
                dest_path.unlink() # Clean up partial download
            raise RuntimeError(f"Failed to download model: {e}")



from pathlib import Path
from monocr_onnx import read_image_with_accuracy

project_root = Path(__file__).resolve().parent.parent.parent
image_path = project_root / "data/images/test_0005_h71.png"

# Dummy ground truth for testing flow
ground_truth = "ဂကောံမန်နာနာတံထံက်ပၚ်သြန်ကုသ္ိကၟိန်ညးဒးဒုၚ်ပန်ပ္ဍဲကွာန်ပါၚ်မၚ်ဂၠန်(၂)တၠ"

print(f"Reading image: {image_path}")
text, accuracy = read_image_with_accuracy(image_path, ground_truth)

print("Recognized Text:")
print(text)
print("-" * 20)
print(f"Accuracy: {accuracy:.2f}%")



import numpy as np
from PIL import Image

class LineSegmenter:
    def __init__(self, min_line_h=10, smooth_window=3):
        self.min_line_h = min_line_h
        self.smooth_window = smooth_window

    def segment(self, image):
        """
        Segment a document image into text lines.
        Args:
            image (PIL.Image or np.ndarray): Input image.
        Returns:
            list: List of dicts with keys 'img' (PIL.Image) and 'bbox' (x, y, w, h).
        """
        if isinstance(image, Image.Image):
            img_pil = image
            if img_pil.mode != 'L':
                img_pil = img_pil.convert('L')
            img_arr = np.array(img_pil)
        else:
            img_arr = image
            img_pil = Image.fromarray(img_arr)
            if img_pil.mode != 'L':
                img_pil = img_pil.convert('L')
                img_arr = np.array(img_pil)

        height, width = img_arr.shape

        # 1. Binarize / Thresholding for Projection
        # Text is dark (< 128). We want pixel count of text.
        # binary = 1 where text, 0 where background.
        binary = (img_arr < 128).astype(np.int32)
        
        # 2. Horizontal Projection Profile
        hist = np.sum(binary, axis=1) # Sum along width (axis 1) -> shape (height,)

        # 3. Smoothing
        if self.smooth_window > 1:
            kernel = np.ones(self.smooth_window) / self.smooth_window
            # mode='same' returns output of same length as input
            smoothed_hist = np.convolve(hist, kernel, mode='same')
        else:
            smoothed_hist = hist

        # 4. Gap Detection
        non_zero_vals = smoothed_hist[smoothed_hist > 0]
        if len(non_zero_vals) == 0:
            return []

        mean_density = np.mean(non_zero_vals)
        gap_threshold = mean_density * 0.05
        
        results = []
        start = None
        
        for y in range(height):
            is_text = smoothed_hist[y] > gap_threshold
            
            if is_text and start is None:
                start = y
            elif not is_text and start is not None:
                end = y
                if (end - start) >= self.min_line_h:
                    self._extract_line(img_pil, img_arr, width, height, start, end, results)
                start = None
                
        if start is not None and (height - start) >= self.min_line_h:
            self._extract_line(img_pil, img_arr, width, height, start, height, results)
            
        return results

    def _extract_line(self, img_pil, img_arr, width, height, r_start, r_end, results):
        # Find horizontal bounds within strip
        # strip is img_arr[r_start:r_end, :]
        strip = img_arr[r_start:r_end, :]
        
        # Find x where col sum > 0 (checking binary text pixels)
        strip_binary = (strip < 128)
        col_sum = np.sum(strip_binary, axis=0) # shape (width,)
        
        non_empty_cols = np.where(col_sum > 0)[0]
        
        if len(non_empty_cols) == 0:
            return
            
        x_min = non_empty_cols[0]
        x_max = non_empty_cols[-1]
        
        # Add padding
        pad = 4
        y1 = max(0, r_start - pad)
        y2 = min(height, r_end + pad)
        x1 = max(0, x_min - pad)
        x2 = min(width, x_max + pad)
        
        w = x2 - x1
        h = y2 - y1
        
        crop = img_pil.crop((x1, y1, x2, y2))
        
        results.append({
            'img': crop,
            'bbox': (int(x1), int(y1), int(w), int(h))
        })
